<?php

if ( !class_exists('wsVoteForPlugins') ):

//TODO: Test if cookie file is really deleted
//TODO: Test in IE

class wsVoteForPlugins {
	
	var $options;
	var $options_name = 'ws_vote_for_plugins';
	var $defaults;
	
	var $plugin_info; //An instance of wsPluginInfo used to store data about plugin ratings/compatibility
	
  /**
   * Class constructor. Loads settings & plugin data and sets up hooks.
   *
   * @return void
   */
	function wsVoteForPlugins() {
		$this->defaults = array(
			'username' => '',
			'password' => '',
		);
		
		//Since settings are saved in user metadata, they can only be loaded after 
		//the user info has been initialized.
		add_action('init', array(&$this, 'init')); 
		
		add_action('admin_menu', array(&$this, 'admin_menu'));
		add_action('admin_print_styles-plugins.php', array(&$this, 'load_voting_css') );
		add_action('admin_print_scripts-plugins.php', array(&$this, 'load_voting_scripts') );
		add_action('admin_notices', array(&$this, 'admin_notices'));
		add_action('admin_footer', array(&$this, 'gather_plugin_data'));
	}
	
  /**
   * Load settings and set up hooks that depend on the current user. 
   *
   * @return void
   */
	function init(){
		$this->load_options();
		$this->plugin_info = new wsPluginInfo('ws_wp.org_plugin_data');
		
		if ( current_user_can('activate_plugins') && !empty($this->options['username']) && !empty($this->options['password']) ){
			add_filter('plugin_row_meta', array(&$this, 'plugin_row_meta'), 10, 3);
		}
	}
	
  /**
   * Set up the plugin's admin menu.
   *
   * @return void
   */
	function admin_menu(){
		add_submenu_page(
			'plugins.php',
			'WordPress.org Account Details', 
			'WordPress.org Account', 
			'activate_plugins', 
			'vote-for-plugins', 
			array(&$this, 'page_options')
		);
	}
	
  /**
   * Enqueue the plugin's mighty CSS file 
   *
   * @return void
   */
	function load_voting_css(){
		wp_enqueue_style('vfp-stars', plugin_dir_url(__FILE__) . 'css/voting.css');
	}
	
  /**
   * Enqueue the plugin's JavaScript on the "Plugins" tab 
   *
   * @return void
   */
	function load_voting_scripts(){
		wp_enqueue_script('vfp-button-helper', plugin_dir_url(__FILE__) . 'js/button-helper.js', array('jquery'));
	}
	
  /**
   * Print the voting & rating UI for the current plugin row
   *
   * @param array $plugin_meta Current entries in the plugin metadata row.
   * @param string $plugin_file
   * @param array $plugin_data
   * @return array New metadata entries.
   */
	function plugin_row_meta($plugin_meta, $plugin_file, $plugin_data){
		//Only show the voting links for plugins that have a version number
		//and are listed in the WordPress.org plugin directory.
		if ( isset($plugin_data['Version']) && $this->plugin_info->listed_in_directory($plugin_file, true) ){
			$current_vote = $this->plugin_info->get_current_vote($plugin_file, $plugin_data['Version']);
			
			$voting_url = 'plugins.php?page=vote-for-plugins&noheader=1&action=vote_compatibility&compatible=%d&plugin=%s&plugin_version=%s';
			
			//A link for voting the plugin as working
			$vote_works_url = sprintf(
				$voting_url,
				1,
				urlencode($plugin_file),
				urlencode($plugin_data['Version'])
			);
			$vote_works_url = wp_nonce_url($vote_works_url, 'vote-plugin-compatibility');
			
			$works = sprintf(
				'<div class="button vfp-button"><a href="%s" class="works %s">Works</a></div>',
				admin_url($vote_works_url),
				($current_vote === true)?'current-vote':''
			);
			
			//A link for voting the plugin as broken
			$vote_broken_url = sprintf(
				$voting_url,
				0,
				urlencode($plugin_file),
				urlencode($plugin_data['Version'])
			);
			$vote_broken_url = wp_nonce_url($vote_broken_url, 'vote-plugin-compatibility');
			
			$broken = sprintf(
				'<div class="button vfp-button"><a href="%s" class="broken %s">Broken</a></div>',
				admin_url($vote_broken_url),
				($current_vote === false)?'current-vote':''
			);
			
			//The starry rating widget
			$ratings = $this->plugin_info->get_current_rating($plugin_file, 'all');
			$average_rating_widget = $this->star_widget($ratings['user_rating'], $plugin_file);
			
			//Assemble the meta entry. It's "all-in-one" because it looks better that way
			//when it's floated to the right.
			$my_meta = array(
				$average_rating_widget,
				$works,
				$broken,
				//$ratings['user_rating'] . ' / ' . $ratings['average_rating'],
			);
			$my_meta = '<div class="vfp-container">'. implode(' ', $my_meta) .'</div>';
			
			$plugin_meta[] = $my_meta;
		}
		
		return $plugin_meta;
	}
	
  /**
   * Get the voting widget HTML.
   *
   * @param float $rating The current rating, on a scale of 0 to 100.
   * @param string $plugin (Optional) If set, the widget will be interactive & allow rating this plugin. Otherwise, it will be a static display. 
   * @return string The widget HTML.
   */
	function star_widget($rating, $plugin = ''){
		$select = !empty($plugin)?' select':'';
		$rating_titles = array(
			1 => 'Poor',
			2 => 'Works',
			3 => 'Good',
			4 => 'Great',
			5 => 'Excellent'
		);
		
		$rating_widget = '<div class="star-holder%s"><div class="star star-rating%s" style="width: %.1f%%"></div>';
		$rating_widget = sprintf(
			$rating_widget,
			$select,
			$select,
			$rating
		);
		$star_url = plugin_dir_url(__FILE__) . 'css/star.gif';
		
		$base_rating_url = 'plugins.php?page=vote-for-plugins&noheader=1&action=rate&plugin=%s&rating=%d';
		
		for($star_num = 5; $star_num > 0; $star_num--){
			$image = sprintf(
				'<img src="%s" alt="%d stars" />',
				$star_url, 
				$star_num
			);
			
			if ( !empty($plugin) ){
				$rating_url = sprintf(
					$base_rating_url,
					$plugin,
					$star_num
				);
				$rating_url = admin_url($rating_url);
				$rating_url = wp_nonce_url($rating_url, 'rate-plugin');
				
				$title = str_repeat('*', $star_num) . ' ' . $rating_titles[$star_num];
				$image = sprintf(
					'<a href="%s" title="%s">%s</a>',
					$rating_url,
					$title,
					$image					
				);
			}
			
			$rating_widget .= sprintf(
				'<div class="star star%d%s">%s</div>',
				$star_num,
				$select,
				$image
			);
		}
		$rating_widget .= '</div>';
		
		return $rating_widget;
	}
	
  /**
   * Settings page. 
   * 
   * Also handles the actual voting & rating requests.
   *
   * @return void
   */
	function page_options(){
		if ( !current_user_can('activate_plugins') ){
			return;
		}
		
		$allowed_actions = array('vote_compatibility', 'rate');
		$action = isset($_GET['action'])?$_GET['action']:'';
		if ( !in_array($action, $allowed_actions) ){
			$action = '';
		}
		
		if ( $action == 'vote_compatibility' ){
			//Vote for plugin compatibility.
			
			check_admin_referer('vote-plugin-compatibility');
			
			$compatible = intval($_GET['compatible']) !== 0;
			$plugin = $_GET['plugin'];
			$plugin_version = $_GET['plugin_version'];
			
			//Submit the vote
			$rez = $this->vote($plugin, $compatible, $plugin_version);
			
			//Figure out the URL of the page the user was on previously so we can redirect back to it
			$previous_page = wp_get_referer();
			if ( !$previous_page ){
				$previous_page = admin_url('plugins.php');
			}
			$previous_page = remove_query_arg(
				array('deactivate', 'activate', 'voting_success', 'rating_success', 'error_code', 'error_message'),
				$previous_page
			);
			//Pass back the vote info. Useful for building a detailed confirmation message.
			$previous_page = add_query_arg(
				array(
					'compatible' => $compatible?1:0,
					'plugin' => urlencode($plugin),
					'plugin_version' => urlencode($plugin_version),
				),
				$previous_page 
			);
			
			if ( is_wp_error($rez) ){
				//Redirect back with an error flag & message
				$previous_page = add_query_arg(
					array(
						'voting_success' => 0,
						'error_code' => urlencode($rez->get_error_code()),
						'error_message' => urlencode($rez->get_error_message()),
					),
					$previous_page
				);
			} else {
				//Redirect back with a success flag
				$previous_page = add_query_arg(
					array(
						'voting_success' => 1,
					),
					$previous_page
				);
			}
			
			wp_redirect($previous_page);
			die();
			
		} elseif ($action == 'rate') {
			//Rate a plugin on the scale of 1 to 5 stars.
			
			check_admin_referer('rate-plugin');
			
			$plugin = $_GET['plugin'];
			$rating = intval($_GET['rating']);
			
			//Submit the rating
			$rez = $this->rate($plugin, $rating);
			
			//Figure out the referer URL sans superfluous arguments
			$previous_page = wp_get_referer();
			if ( !$previous_page ){
				$previous_page = admin_url('plugins.php');
			}
			$previous_page = remove_query_arg(
				array('deactivate', 'activate', 'voting_success', 'rating_success', 'error_code', 'error_message'),
				$previous_page
			);
			//Pass back the rating for use in the confirmation message.
			$previous_page = add_query_arg(
				'rating', $rating, $previous_page
			);
			
			if ( is_wp_error($rez) ){
				//Error, redirect back with an error flag & message
				$previous_page = add_query_arg(
					array(
						'rating_success' => 0,
						'error_code' => urlencode($rez->get_error_code()),
						'error_message' => urlencode($rez->get_error_message()),
						'plugin' => urlencode($plugin),
					),
					$previous_page
				);
			} else {
				//Success, set the success flag
				$previous_page = add_query_arg(
					array(
						'rating_success' => 1,
						'plugin' => $plugin,
					),
					$previous_page
				);
			}
			
			wp_redirect($previous_page);
			die();
		} 
		
		//Save WordPress.org account details
		if(isset($_POST['submit']) && current_user_can('activate_plugins')) {
			check_admin_referer('vote-for-plugins-settings');
			
			$this->options['username'] = $_POST['username'];
			$this->options['password'] = $_POST['password'];
			$this->save_options();
			
			//Go back to the settings page
			$options_page = wp_get_referer();
			$options_page = add_query_arg(
				'updated', 'true', $options_page
			);
			
			wp_redirect($options_page);
			die();
		}
		
		if ( isset($_GET['updated']) && ($_GET['updated'] == 'true') ) {
			echo "<div id='setting-error-settings_updated' class='updated settings-error'><p><strong>Settings saved.</strong></p></div>";
		}
		
		?>
		
		<div class="wrap"><h2>WordPress.org Account Details</h2>
		<form name="cloaking_options" method="post" action="<?php echo esc_attr(admin_url('plugins.php?page=vote-for-plugins&noheader=1')); ?>"> 
		<?php wp_nonce_field('vote-for-plugins-settings'); ?>
		
		<table class="form-table"> 

			<tr valign="top">
			<th scope="row"><label for="username">Username</label></th>
			<td><input name="username" type="text" id="username" value="<?php echo esc_attr($this->options['username']); ?>" class="regular-text" /></td>
			</tr>
			 
			<tr valign="top"> 
			<th scope="row"><label for="password">Password</label></th>
			<td><input name="password" type="password" id="password" value="<?php echo esc_attr($this->options['password']); ?>" class="regular-text" /></td> 
			</tr>
			
		</table>
		
		<p class="submit"><input type="submit" name="submit" class='button-primary' value="<?php _e('Save Changes'); ?>" /></p>	
		</form>
		</div>
		
		<?php
	}
	
  /**
   * Print error/confirmation notice about the results of submitting a rating or compatibility vote.
   *
   * Executed by the 'admin_notices' action. 
   *
   * @return void
   */
	function admin_notices(){
		global $pagenow, $wp_version;
		
		$settings_page = isset($_GET['page']) && ($_GET['page'] == 'vote-for-plugins');
		if ( !$settings_page && (empty($this->options['username']) || empty($this->options['password'])) ){
			$message = sprintf(
				'You need to <a href="%s">enter your account info</a> to be able to vote on plugins.',
				admin_url('plugins.php?page=vote-for-plugins')
			);
			echo '<div id="message" class="updated"><p>'.$message.'</p></div>';
		}
		
		if ( $pagenow != 'plugins.php' ){
			return;
		}
		
		if ( isset($_GET['voting_success']) ){
			$success = intval($_GET['voting_success']) == 1;
			$compatible = intval($_GET['compatible']) == 1;
			$plugin_version = $_GET['plugin_version'];
			$plugin_file = $_GET['plugin'];
			
			if ($success){
				$installed_plugins = get_plugins();
				$plugin = $installed_plugins[$plugin_file];
				
				$message = sprintf(
					'You have marked %s %s as <strong>%s</strong> on WordPress %s.',
					$plugin['Name'],
					$plugin_version,
					$compatible?'working':'broken',
					$wp_version
				);
				
				echo '<div id="message" class="updated"><p>'.$message.'</p></div>';
			} else {
				echo '<div id="message" class="error"><p>'.stripslashes($_GET['error_message']).'</p></div>';
			}
			
		} elseif ( isset($_GET['rating_success']) ){
			$success = intval($_GET['rating_success']) == 1;
			$rating = intval($_GET['rating']);
			
			if ($success){
				$installed_plugins = get_plugins();
				$plugin = $installed_plugins[$_GET['plugin']];
				
				$message = sprintf(
					_n(
						'You have rated "%s" with %d star.',
						'You have rated "%s" with %d stars.',
						$rating
					),
					$plugin['Name'],
					$rating
				);
				
				echo '<div id="message" class="updated"><p>'.$message.'</p></div>';
			} else {
				echo '<div id="message" class="error"><p>'.stripslashes($_GET['error_message']).'</p></div>';
			}
			
		}
		
	}
	
  /**
   * Load plugin settings.
   *
   * @return void
   */
	function load_options(){
		$user = wp_get_current_user();
		if ( !empty($user) ){
			$this->options = get_metadata('user', $user->ID, $this->options_name, true);
		}
	    
	    if( !is_array( $this->options ) ){
	        $this->options = $this->defaults;
	    } else {
	        $this->options = array_merge( $this->defaults, $this->options );
	    }
	}
	
  /**
   * Save plugin settings.
   *
   * @return void
   */
	function save_options(){
		$user = wp_get_current_user();
		if ( !empty($user) ){
			update_metadata('user', $user->ID, $this->options_name, $this->options);
		}
		//update_option($this->options_name, $this->options);
	}
	
  /**
   * Vote about a plugin's compatibility with a specific WP version.
   *
   * @param string $plugin_file
   * @param bool $compatible
   * @param string $plugin_version
   * @param string $wordpress_version Defaults to the current WP version.
   * @return bool|WP_Error
   */
	function vote($plugin_file, $compatible, $plugin_version, $wordpress_version = null){
		global $wp_version;
		
		if ( is_null($wordpress_version) ){
			$wordpress_version = $wp_version;
		}
		
		if ( empty($this->options['username']) || empty($this->options['password']) ){
			return new WP_Error(
				'no_account_data',
				"You need to enter your WordPress.org account data in <em>Plugins -&gt; WordPress.org Account</em> first!"
			);
		}
		
		$voter = new wsPluginVoter($this->options['username'], $this->options['password']);
		$rez = $voter->vote($plugin_file, $compatible, $plugin_version, $wordpress_version);
		$voter->done();
		
		if ( is_wp_error($rez) ){
			
			if ( $rez->get_error_code() == 'not_in_directory' ){
				//Remember that this plugin is not listed in the WordPress.org directory
				//so that we don't try to vote for it again.
				$this->plugin_info->set_directory_status($plugin_file, false);
			}
			
			return $rez;
		} else {
			$this->plugin_info->set_directory_status($plugin_file, true);
			$this->plugin_info->set_current_vote($plugin_file, $compatible, $plugin_version, $wordpress_version);
			return true;
		}
	}
	
  /**
   * Rate a plugin with X stars.
   *
   * @param string $plugin Plugin filename, e.g. "my-plugin/my-plugin.php"
   * @param int $rating Star rating, 1 - 5.
   * @return bool|WP_Error
   */
	function rate($plugin, $rating){
		if ( empty($this->options['username']) || empty($this->options['password']) ){
			return new WP_Error(
				'no_account_data',
				"You need to enter your WordPress.org account data in <em>Plugins -&gt; WordPress.org Account</em> first!"
			);
		}
		
		$voter = new wsPluginVoter($this->options['username'], $this->options['password']);
		$rez = $voter->rate($plugin, $rating);
		$voter->done();
		
		if ( is_wp_error($rez) ){
			
			if ( $rez->get_error_code() == 'not_in_directory' ){
				//Remember that this plugin is not listed in the WordPress.org directory
				//so that we don't try to vote for it again.
				$this->plugin_info->set_directory_status($plugin, false);
			}
			
			return $rez;
		} else {
			$this->plugin_info->set_directory_status($plugin, true);
			//Ratings are internally stored as a floating point number between 0 and 100.
			$this->plugin_info->set_current_rating($plugin, null, $rating*20); 
			return true;
		}
	}
	
  /**
   * Update ratings and compatibility votes for all installed plugins listed in 
   * the WordPress.org plugin directory.
   *
   * The execution time of this function is limited to avoid slowing down normal WP operation,
   * so you may need to perform several runs to collect data about all plugins. 
   *
   * @return void
   */
	function gather_plugin_data(){
		if ( empty($this->options['username']) || empty($this->options['password']) ) {
			return;
		}
		
		//The data gathering routine is run on the Dashboard, so we don't want to run
		//it for too long lest the Dashboard appears laggy. Running over the exec. time
		//limit set in php.ini would be even worse.
		$max_execution_time = 20;
		$max_execution_time = min(array(
			$max_execution_time,
			intval(ini_get('max_execution_time')) - 5,
		));
		
		if ( $max_execution_time < 1 ){
			$max_execution_time = 1;
		}
		
		
		$start_time = time();
		
		$recheck_period = 3*60*60*24; //3 days
		
		//Find plugins that need to be checked
		$installed_plugins = get_plugins();
		$need_checking = array();
		foreach($installed_plugins as $plugin => $plugin_data){
			$last_check = $this->plugin_info->get_plugin_info($plugin, 'last_checked'); 
			$recheck = (time() - $last_check) > $recheck_period;
			
			if ( $recheck ){
				$need_checking[$plugin] = $plugin_data;
			}
		}
		
		if ( empty($need_checking) ){
			return;
		}
		
		$voter = new wsPluginVoter($this->options['username'], $this->options['password']);
		$rez = $voter->log_in();
		if ( is_wp_error($rez) ){
			return;
		}
		
		//Collect directory info about the selected plugins
		foreach($need_checking as $plugin => $plugin_data){
			if ( (time() - $start_time) >= $max_execution_time ){
				break;
			}
			$this->plugin_info->set_plugin_info($plugin, 'last_checked', time(), false);
			
			$page = $voter->fetch_directory_page($plugin);
			
			//Make a note of whether the plugin is in the directory
			if ( is_wp_error($page) ){
				if ( $page->get_error_code() == 'not_in_directory' ){
					$this->plugin_info->set_directory_status($plugin, false, false);
				}
				//Skip problematic/unlisted plugins; we can't get any ratings or
				//compatibility votes for them anyway.
				continue; 
			}
			$this->plugin_info->set_directory_status($plugin, true, false);
			
			//Extract compatibility votes & ratings
			$votes = $voter->extract_plugin_votes($page);
			foreach($votes['user_votes'] as $wordpress_version => $plugin_versions){
				foreach($plugin_versions as $plugin_version => $compatible){
					$this->plugin_info->set_current_vote($plugin, $compatible, $plugin_version, $wordpress_version, false);
				}
			}
			
			//Store star ratings
			$this->plugin_info->set_current_rating($plugin, $votes['average_rating'], $votes['user_rating'], false);
			
		}
		$voter->done();
		
		//Clean out info about uninstalled plugins
		//$this->plugin_info->cleanup();
		//Save changes
		$this->plugin_info->save_data();
	}
} 

endif;

$ws_vote_for_plugins = new wsVoteForPlugins();

?>